# The MIT License (MIT)
#
# Copyright (C) 2025 Fabrício Barros Cabral
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
from abc import ABC, abstractmethod
from datetime import datetime, timedelta, timezone

from playwright.sync_api import sync_playwright

import arana.page
from arana.console import Console, StdConsole
from arana.page import Page, PwPage


class Browser(ABC):
    @abstractmethod
    def open(self, *, headless: bool = True) -> None:
        pass

    @abstractmethod
    def close(self) -> None:
        pass

    @abstractmethod
    def name(self) -> str:
        pass

    @abstractmethod
    def page(self, url: str) -> Page:
        pass


class Chromium(Browser):
    def __init__(self) -> None:
        self.__playwright = sync_playwright().start()
        self.__pages: list[Page] = []

    def open(self, *, headless: bool = True) -> None:
        self.__rocket = self.__playwright.chromium.launch(headless=headless)

    def close(self) -> None:
        for page in self.__pages:
            page.close()
        self.__rocket.close()
        self.__playwright.stop()

    def name(self) -> str:
        return "Chromium"

    def page(self, url: str) -> Page:
        page = PwPage(self.__rocket, url)
        self.__pages.append(page)
        return page


class Firefox(Browser):
    def __init__(self) -> None:
        self.__playwright = sync_playwright().start()
        self.__pages: list[Page] = []

    def open(self, *, headless: bool = True) -> None:
        self.__rocket = self.__playwright.firefox.launch(headless=headless)

    def close(self) -> None:
        for page in self.__pages:
            page.close()
        self.__rocket.close()
        self.__playwright.stop()

    def name(self) -> str:
        return "Firefox"

    def page(self, url: str) -> Page:
        page = PwPage(self.__rocket, url)
        self.__pages.append(page)
        return page


class Webkit(Browser):
    def __init__(self) -> None:
        self.__playwright = sync_playwright().start()
        self.__pages: list[Page] = []

    def open(self, *, headless: bool = True) -> None:
        self.__rocket = self.__playwright.webkit.launch(headless=headless)

    def close(self) -> None:
        for page in self.__pages:
            page.close()
        self.__rocket.close()
        self.__playwright.stop()

    def name(self) -> str:
        return "Webkit"

    def page(self, url: str) -> Page:
        page = PwPage(self.__rocket, url)
        self.__pages.append(page)
        return page


class Logged(Browser):
    def __init__(
        self, browser: Browser, console: Console = StdConsole()
    ) -> None:
        self.__origin = browser
        self.__console = console

    def open(self, *, headless: bool = True) -> None:
        zone = timezone(timedelta(hours=-3))
        timestamp = datetime.now(zone).strftime("%H:%M:%S.%f")
        self.__console.log(f"[{timestamp}] Opening browser {self.name()}... ")
        self.__origin.open(headless=headless)
        self.__console.logln("done.")

    def close(self) -> None:
        zone = timezone(timedelta(hours=-3))
        timestamp = datetime.now(zone).strftime("%H:%M:%S.%f")
        self.__console.log(f"[{timestamp}] Closing browser {self.name()}... ")
        self.__origin.close()
        self.__console.logln("done.")

    def name(self) -> str:
        return self.__origin.name()

    def page(self, url: str) -> Page:
        zone = timezone(timedelta(hours=-3))
        timestamp = datetime.now(zone).strftime("%H:%M:%S.%f")
        self.__console.log(f"[{timestamp}] Creating a new page... ")
        page = arana.page.Logged(self.__origin.page(url), self.__console)
        self.__console.logln("done.")
        return page


class Headed(Browser):
    def __init__(self, browser: Browser) -> None:
        self.__origin = browser

    def open(self, *, headless: bool = True) -> None:
        self.__origin.open(headless=False)

    def close(self) -> None:
        self.__origin.close()

    def name(self) -> str:
        return self.__origin.name()

    def page(self, url: str) -> Page:
        return self.__origin.page(url)



# The MIT License (MIT)
#
# Copyright (C) 2025 Fabrício Barros Cabral
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
import random
import time

"""Generate random integer numbers."""


class RandomInt:
    def value(self, start: int, end: int) -> int:
        """Generate a random integer number between two integer numbers inclusive.

        Args:
            start (int): start number interval
            end (int): end number interval

        Returns:
            int: a random integer number between start and end inclusive

        """
        return random.randrange(start, end + 1)

    def values(self, start: int, end: int) -> list[int]:
        """Generate a list of integer random numbers between two integer numbers inclusive.

        Args:
            start (int): start number interval
            end (int): end number interval

        Returns:
            list[int]: a list of integer numbers between start and end inclusive

        """
        return random.sample(range(start, end + 1), end - start + 1)


class RandomWait:
    def __init__(self, min_secs: int = 1, max_secs: int = 10) -> None:
        self.__min_secs = min_secs
        self.__max_secs = max_secs

    def run(self) -> None:
        time.sleep(random.randrange(self.__min_secs, self.__max_secs + 1))



# The MIT License (MIT)
#
# Copyright (C) 2025 Fabrício Barros Cabral
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
from abc import ABC, abstractmethod

from arana.html import Html


class Response(ABC):
    @abstractmethod
    def status(self) -> int:
        pass

    @abstractmethod
    def html(self) -> Html:
        pass

    @abstractmethod
    def url(self) -> str:
        pass


class PwResponse(Response):
    def __init__(self, status: int, html: Html, url: str) -> None:
        self.__status = status
        self.__html = html
        self.__url = url

    def status(self) -> int:
        return self.__status

    def html(self) -> Html:
        return self.__html

    def url(self) -> str:
        return self.__url



# The MIT License (MIT)
#
# Copyright (C) 2025 Fabrício Barros Cabral
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
import re

from arana.browser import Chromium, Firefox, Logged, Webkit
from arana.console import FakeConsole


def test_chromium() -> None:
    browser = Chromium()
    browser.open()
    browser.close()


def test_firefox() -> None:
    browser = Firefox()
    browser.open()
    browser.close()


def test_webkit() -> None:
    browser = Webkit()
    browser.open()
    browser.close()


def test_logged() -> None:
    console = FakeConsole()
    browser = Logged(Chromium(), console)
    browser.open()
    browser.close()
    assert re.search(
        r"\[[0-9]{2}:[0-9]{2}:[0-9]{2}\.[0-9]{6}] Opening browser Chromium... "
        r"done.\n"
        r"\[[0-9]{2}:[0-9]{2}:[0-9]{2}\.[0-9]{6}] Closing browser Chromium... "
        r"done.\n",
        console.stderr(),
    )



# The MIT License (MIT)
#
# Copyright (C) 2025 Fabrício Barros Cabral
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
from fake_https_server.request import ContentGet
from fake_https_server.server import Daemon, FakeHttpsServer

from arana.browser import Chromium
from arana.content import ContentText


def test_refine() -> None:
    success_ok = 200
    text = "It works!"
    browser = Chromium()
    browser.open()
    server = Daemon(FakeHttpsServer(ContentGet(text)))
    server.start()
    url = f"https://localhost:{server.port()}"
    page = browser.page(url)
    response = page.open()
    result = ContentText(url).refine(response)
    assert response.status() == success_ok
    assert result["text"] == text
    page.close()
    server.stop()
    browser.close()



# The MIT License (MIT)
#
# Copyright (C) 2025 Fabrício Barros Cabral
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
from __future__ import annotations

from abc import ABC, abstractmethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from playwright.sync_api import Locator


class Element(ABC):
    @abstractmethod
    def text(self) -> str:
        pass

    @abstractmethod
    def texts(self) -> list[str]:
        pass

    @abstractmethod
    def visible(self) -> bool:
        pass

    @abstractmethod
    def click(self) -> None:
        pass

    @abstractmethod
    def count(self) -> int:
        pass

    @abstractmethod
    def nth(self, index: int) -> Element:
        pass

    @abstractmethod
    def attribute(self, name: str) -> str:
        pass


class PwElement(Element):
    def __init__(self, locator: Locator) -> None:
        self.__locator = locator

    def text(self) -> str:
        txt = ""
        if self.__locator.count() > 1:
            txt = self.__locator.nth(0).inner_text()
        else:
            txt = self.__locator.inner_text()
        return txt

    def texts(self) -> list[str]:
        return self.__locator.all_inner_texts()

    def visible(self) -> bool:
        return self.__locator.is_visible()

    def click(self) -> None:
        self.__locator.click()

    def count(self) -> int:
        return self.__locator.count()

    def nth(self, index: int) -> Element:
        return PwElement(self.__locator.nth(index))

    def attribute(self, name: str) -> str:
        attribute: str | None = self.__locator.get_attribute(name)
        if attribute is None:
            attribute = ""
        return attribute
